/*----------------------------------------------------------------------------------------*/
/* reg.c(p) - demo source of a serialization plugin for GEM Setup (reg.ovl)					*/
/* written by Joachim Fornallaz - last update on the 17th of september 1999					*/
/*																														*/
/* Notes: 1) The startup code must be able to handle the ARGV-parameter mechanism			*/
/*			 2) Set tab to 3																						*/
/*----------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------*/
/* global includes																								*/
/*----------------------------------------------------------------------------------------*/

#include <portab.h>

#include <mgx_dos.h>															/* maybe change to tos.h */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/*----------------------------------------------------------------------------------------*/
/* global variables/defines																					*/
/*----------------------------------------------------------------------------------------*/

#ifndef	BOOL
#define	BOOL	int
#endif
#ifndef	FALSE
#define	FALSE	0
#endif
#ifndef	TRUE
#define	TRUE	1
#endif

#define	DEMO_SUPPORT	TRUE												/* change to "TRUE" if 
																					demo mode is supported */
#define	MAGIC_STRING	"SNCHCK02"

BOOL		gl_demomode = FALSE;					

/*----------------------------------------------------------------------------------------*/
/* functions																										*/
/*----------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------*/
/* Demo mode checking																							*/
/* Returncode:	TRUE if demo mode is supported, else FALSE											*/
/* Note: This function doesn't have to be changed.														*/
/*----------------------------------------------------------------------------------------*/
BOOL	check_demo( char *sn )
{
	if( strcmp( sn, "DEMO" ) == 0 ) 
	{
		if( DEMO_SUPPORT )
		{
			gl_demomode = TRUE;
		}
		else
			return FALSE;
	}
	return TRUE;
}

/*----------------------------------------------------------------------------------------*/
/* Checks, if a parameter is "empty", that means if it is equal "-" 								*/
/* Returncode:	TRUE if the string is empty, else FALSE												*/
/*----------------------------------------------------------------------------------------*/
BOOL	is_empty( char *parameter )
{
	if( strcmp( parameter, "-" ) == 0 )
		return TRUE;
	else
		return FALSE;
}

/*----------------------------------------------------------------------------------------*/
/* Serial number checking																						*/
/* Returncode:	TRUE if the number/string is valid, else FALSE										*/
/* Note: THIS IS SAMPLE CODE!																					*/
/*----------------------------------------------------------------------------------------*/
BOOL	sn_check( char *sn )
{
	if( strcmp( sn, "ATARI" ) == 0 )
		return TRUE;
	else
	{
		puts("The valid SN is \"ATARI\"");
		return FALSE;
	}
}

/*----------------------------------------------------------------------------------------*/
/* Serialize the program file 																				*/
/* Returncode:	-13 (file not found), -12 (serialization error), -1 (general error), 0 (OK)*/
/* Note: THIS IS SAMPLE CODE!																					*/
/*----------------------------------------------------------------------------------------*/
int	sn_file( char *file, char *sn, char *name, char *org, char *town )
{
	long	lret;
	int	handle;
	int	ret = 0;															/* Returncode */

	lret = Fopen( file, FO_RW );

	if( lret < 0 )
	{
		if( lret == EFILNF )												/* File not found */
			ret = -13;
		else																	/* General error */
			ret = -1;
	}
	else
	{
		handle = (WORD) lret;
		lret = 0;
		
		/* SERIALIZATION CODE HERE */
		/* lret = Fopen/Fseek/Fwrite */
		
		Cconws( "[SAMPLE] Serialize file: " );						/* DEMO CODING */
		Cconws( file );
		Cconws( "\r\n" );
		
		if( lret < 0 )														/* Serialization error */
			ret = -12;
			
		Fclose( handle );
	}

	return ret;
}

/*----------------------------------------------------------------------------------------*/
/* main function - serial number checking																	*/
/*																														*/
/* Return values for GEM Setup (since V1.53), magic string is now "SNCHCK02":					*/
/*																														*/
/*		  0 : Everything is OK, the installation can go on												*/
/*		 -1 : General error																						*/
/*	[!] -2 :	Wrong magic string in argv[1], must be "SNCHCK02"										*/
/*		 -3 : Wrong number of arguments, argc must be greater than 6 								*/
/* New -7 : Incomplete Information (Town)																	*/
/* New -8 : Incomplete Information (Organitzation)														*/
/* New -9 : Incomplete Information (Name)																	*/
/*		-10 : The serial number is wrong, GEM Setup will ask the user to re-enter the code	*/
/*		-11 : The demo mode is not supported by this program											*/
/*		-12 : A program file could not be serialized														*/
/*		-13 : A program file could not be found															*/
/*----------------------------------------------------------------------------------------*/
void	main( int argc, char *argv[], char *envp[] )
{
	int	ret = 0;															/* Returncode */

	if( argc > 6 )
	{
		if( strcmp( argv[1], MAGIC_STRING ) != 0 )				/* Check magic string */
			Pterm(-2);

	/*	if( is_empty( argv[3] ) )										/* Name-field is blank */
			Pterm(-9);
	*/
	/*	if( is_empty( argv[4] ) )										/* Org-field is blank */
			Pterm(-8);
	*/
	/*	if( is_empty( argv[5] ) )										/* Town-field is blank */
			Pterm(-7);
	*/
		if( check_demo( argv[2] ) == FALSE )						/* Check demo mode */
			Pterm(-11);

		if( !gl_demomode && sn_check( argv[2] ) == FALSE )		/* Check serial number */
			Pterm(-10);
	
		if( argv[6][0] != '-' )											/* Serialize a file */
			ret = sn_file( argv[6], argv[2], argv[3], argv[4], argv[5] );
		else
			ret = 0;
	}
	else
		Pterm(-3);															/* Wrong number of arguments */

	Pterm( ret );
}