/********************************************************************
*
* MagiC Device Driver Development Kit
* ===================================
*
* C-Modul
*
*
* Das MDDDK enthlt alle notwendigen Module, um in 'C' einen
* Gertetreiber fr MagiC 3.0 zu erstellen. Im Modul SAMPLE.S
* wird das MagiC Assembler-Interface fr 'C' umgesetzt. Um zu
* allen Compilern kompatibel zu sein, sind alle Funktionen in
* PureC als "cdecl" definiert, d.h. die Parameterbergabe geht
* immer ber den Stack.
*
* Die Include-Dateien mgx_???c.h enthalten cdecl-Prozedur-
* Deklarationen fr den Zugriff ber den "Glue"- Code in sample.s,
* d.h. die Strukturen sind nicht die des Betriebssystems, sondern
* die des "Glue"- Codes. Der "Glue"- Code setzt dann die Prozedur-
* aufrufe fr das Betriebssystem um.
*
* Der Beispieltreiber ignoriert Lese- und Schreibzugriffe, an den
* entsprechenden Stellen sind die eigenen Routinen einzufgen.
*
* Zur detaillierten Beschreibung der Funktionen siehe MGX_DFS.TXT.
*
* Fr die Programmierung eines interruptgesteuerten Gertetreibers
* siehe DEV_LPT1. In DEV_LPT1 sind einige der Gertefunktionen
* in Assembler ausgefhrt, was einen schnelleren und kompakteren
* Code ermglicht.
*
*
* (C) Andreas Kromke, 1994
*
********************************************************************/

#include <string.h>
#include <tos.h>
#include <tosdefs.h>

#include "magx.h"
#include "mgx_xfsc.h"
#include "mgx_dfsc.h"

#ifndef NULL
#define NULL ((void *) 0)
#endif


/******************************************************************
*
* Import aus dem Assemblermodul "sample.s".
*
* _drvr ist die Struktur, die beim Kernel angemeldet wird. Sie
* setzt die Aufrufe von Assembler in 'C' um, d.h. kopiert die
* bergaberegister auf den Stack und ruft die Routinen von drvr
* auf.
* Die Routinen von drvr sind daher als "cdecl" deklariert.
*
* Wenn Routinen in Assembler ausgefhrt werden sollen, kann hier
* in der Struktur "drvr" ein Nullzeiger eingetragen und die
* Funktion vollstndig im Assemblerteil implementiert werden.
*
******************************************************************/

extern MX_DDEV _drvr;


/******************************************************************
*
* gertespezifische Variablen
*
******************************************************************/

MX_KERNEL *kernel;



#pragma warn -par

/******************************************************************
*
* Gert ffnen
*
******************************************************************/

LONG cdecl sample_open( MX_DOSFD *f )
{
	return(E_OK);
}


/******************************************************************
*
* Gert schlieen
*
******************************************************************/

LONG cdecl sample_close( MX_DOSFD *f )
{
	return(E_OK);
}


/******************************************************************
*
* lesen
*
******************************************************************/

LONG cdecl sample_read( MX_DOSFD *f, void *buf,  LONG len )
{
	return(0L);
}


/******************************************************************
*
* schreiben
*
******************************************************************/

LONG cdecl sample_write( MX_DOSFD *f, void *buf,  LONG len )
{
	return(0L);
}


/******************************************************************
*
* Status
*
******************************************************************/

LONG cdecl sample_stat( MX_DOSFD *f, WORD rwflag, void *unsel,
					void *appl)
{
	return(0);
}


/******************************************************************
*
* Dateizeiger positionieren
*
******************************************************************/

LONG cdecl sample_seek( MX_DOSFD *f, LONG where, WORD mode )
{
	return(EACCDN);
}


/******************************************************************
*
* Uhrzeit/Datum der geffneten Datei
*
******************************************************************/

LONG cdecl sample_datime( MX_DOSFD *f, WORD *buf, WORD rwflag)
{
	return(EACCDN);
}


/******************************************************************
*
* Gertespezifische Befehle
*
******************************************************************/

LONG cdecl sample_ioctl( MX_DOSFD *f, WORD cmd, void *buf )
{
	return(EINVFN);
}


/******************************************************************
*
* Gert wird gelscht
*  Gertetreiber aus dem Speicher entfernen (!)
*
******************************************************************/

LONG cdecl sample_delete ( MX_DOSFD *parent, MX_DOSDIR *dir )
{
	/* hier den Treiber abmelden */

	/* ... */

	kernel->Pfree(_BasPag);
	return(E_OK);
}

/******************************************************************
*
* Einzelzeichen einlesen (fr Fgetchar)
*
* mode & 0x0001:    cooked
* mode & 0x0002:    echo mode
*
* Rckgabe: ist i.a. ein Langwort bei CON, sonst ein Byte
*           0x0000FF1A bei EOF
*
******************************************************************/

LONG cdecl sample_getc( MX_DOSFD *f, WORD mode )
{
	return(0L);
}


/******************************************************************
*
* Zeile einlesen (fr Cconrs)
*
* mode & 0x0001:    cooked
* mode & 0x0002:    echo mode
*
* Rckgabe: Anzahl gelesener Bytes oder Fehlercode
*
******************************************************************/

LONG cdecl sample_getline( MX_DOSFD *f, char *buf,
						LONG size, WORD mode )
{
	return(0L);
}


/******************************************************************
*
* Einzelzeichen ausgeben (fr Fputchar)
*
* mode & 0x0001:    cooked
*
* Rckgabe: Anzahl geschriebener Bytes, 4 bei einem Terminal
*
******************************************************************/

LONG cdecl sample_putc( MX_DOSFD *f, WORD mode, LONG val )
{
	return(0L);
}


#pragma warn +par


MX_DDEV drvr =
	{
	sample_open,
	sample_close,
	sample_read,
	sample_write,
	sample_stat,
	sample_seek,
	sample_datime,
	sample_ioctl,
	sample_delete,
	sample_getc,
	sample_getline,
	sample_putc
	};

int main( void )
{
	long errcode;

	/* ggf. Standardfunktionen */
	/*
	_drvr.ddev_datime	= NULL;	/* fr Fdatime */
	_drvr.ddev_ioctl	= NULL;	/* fr Fcntl */
	_drvr.ddev_getc	= NULL;	/* fr Fgetchar */
	_drvr.ddev_getline	= NULL;	/* fr Cconrs */
	_drvr.ddev_putc	= NULL;	/* fr Fputchar */
	*/


	errcode = Dcntl(KER_GETINFO, NULL, 0L);
	if   (errcode > 0L)
		{
		kernel = (MX_KERNEL *) errcode;
		errcode = Dcntl(DEV_M_INSTALL, "u:\\dev\\sample",	(long) &_drvr);
		}
	if	(errcode < 0L)
		{
		return((int) errcode);
		}

	Ptermres(-1L, 0);        /* allen Speicher behalten */
	return(0);
}
