/*
*
* Rumpf einer "shared library"
*
* Andreas Kromke
* 22.10.97
*
*/

#include <portab.h>
#include <tos.h>
#include <tosdefs.h>
#pragma warn -par

typedef void *PD;

char *mem;			/* hier globalen Speicher */

/*****************************************************************
*
* Die init-Funktion wird einmal beim Laden der Bibliothek
* aufgerufen. Dabei luft sie im Proze der Bibliothek,
* d.h. es knnen Dateien geffnet und Speicher angefordert
* werden, die jeweils der Bibliothek gehren.
* Achtung: Auf die dabei geffneten Dateien darf durch die
*          Bibliotheksfunktionen _NICHT_ zugegriffen werden,
*          weil diese im Kontext des Aufrufers laufen.
*
* Achtung: Die init-Funktion luft im Supervisormode, da eine
*          Bibliothek i.a. keinen Userstack hat.
*          Daher darf sie nicht zuviel Stack benutzen (max. 1kB)
*          und nicht zu lange laufen (weil das Multitasking
*          im Supervisormode blockiert ist).
*          Ggf. kann aber ein Userstack alloziert und in den
*          Usermode gewechselt werden.
*
*****************************************************************/

extern LONG cdecl slb_init( void )
{
	mem = Malloc(4096L);
	if	(mem)
		return(E_OK);
	else	return(ENSMEM);
}

/*****************************************************************
*
* Die exit-Funktion wird einmal beim Freigeben der Bibliothek
* aufgerufen. Dabei luft sie im Proze der Bibliothek,
* d.h. es knnen Dateien geffnet und Speicher angefordert
* werden, die jeweils der Bibliothek gehren.
*
* Achtung: Die exit-Funktion luft im Supervisormode, da eine
*          Bibliothek i.a. keinen Userstack hat.
*          Daher darf sie nicht zuviel Stack benutzen (max. 1kB)
*          und nicht zu lange laufen (weil das Multitasking
*          im Supervisormode blockiert ist).
*          Ggf. kann aber ein Userstack alloziert und in den
*          Usermode gewechselt werden.
*
*****************************************************************/

extern void cdecl slb_exit( void )
{
	Mfree(mem);
}


/*****************************************************************
*
* Die open-Funktion wird einmal beim ffnen der Bibliothek
* durch einen Anwenderproze aufgerufen. Dabei luft sie im
* Proze des Aufrufers, d.h. es knnen Dateien geffnet und
* Speicher angefordert werden, die jeweils dem Aufrufer gehren.
*
* Durch den Kernel ist sichergestellt, da jeder Proze die
* Bibliothek nicht mehrmals ffnet und da die Bibliothek immer
* ordnungsgem geschlossen wird.
*
* Achtung: Die open-Funktion luft im Usermode, und zwar mit dem
*          Userstack des Aufrufers. Das heit, da der Aufrufer,
*          auch wenn er im Supervisormode luft, immer einen
*          ausreichend groen usp zur Verfgung stellen mu.
*
*****************************************************************/

extern LONG cdecl slb_open( PD *pd )
{
	return(E_OK);
}


/*****************************************************************
*
* Die close-Funktion wird einmal beim Schlieen der Bibliothek
* durch einen Anwenderproze aufgerufen. Dabei luft sie im
* Proze des Aufrufers, d.h. es knnen Dateien geffnet bzw.
* geschlossen und Speicher angefordert und freigegeben  werden,
* die jeweils dem Aufrufer gehren.
*
* Achtung: Die close-Funktion luft im Usermode, und zwar mit dem
*          Userstack des Aufrufers. Das heit, da der Aufrufer,
*          auch wenn er im Supervisormode luft, immer einen
*          ausreichend groen usp zur Verfgung stellen mu.
*
*****************************************************************/

extern void cdecl slb_close( PD *pd )
{
}


/*****************************************************************
*
* Eine Beispiel-Bibliotheksfunktion.
* Sie wird im Kontext des Aufrufers ausgefhrt, und zwar mit dem
* Stack des Aufrufers (je nach Status usp oder ssp).
*
* Es wird dringend empfohlen, die Funktionen einer SLB nur im
* Usermode aufzurufen, um die Kompatibilitt zu spteren
* Implementationen zu wahren.
*
*****************************************************************/

extern LONG cdecl slb_fn0( PD *pd, LONG fn, WORD nargs, char *s )
{
	Cconws(s);
	Cconws("\r\nTaste: ");
	Cconin();
	return(E_OK);
}
